#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <gl/gl.h>
#include <dmedia/vl.h>

#define YUV_411_FRAME_SIZE 38016

VLServer svr;
VLPath path;
VLNode src, drn;
VLControlValue val;
VLBuffer buffer;
VLInfoPtr info;
char *dataPtr;
int c;
int xsize;
int ysize;
long win;


void init_capture();
void init_dpy();

/* Report errors */
void
error_exit(void)
{
    vlPerror("");
    exit(1);
}

void
main(int argc, char **argv)
{
    FILE *file;
    char buf[102400];
    int i;

    file = fopen("tomek.yuv", "w");
    init_capture();
    init_dpy();
    
    printf("Transfer size: %d\n", vlGetTransferSize(svr, path));
    
    while(1)
      {
	do {
	    sginap(1);		/* wait a tick */
	    info = vlGetNextValid(svr, buffer);
	} while (!info);
    
	/* Get a pointer to the frame */
	dataPtr = vlGetActiveRegion(svr, buffer, info);
		
	/* Write the data to the screen */
	lrectwrite(0,0, xsize-1, ysize-1, (ulong *)dataPtr);

	/* YUYV 422 to YUV 411 convert (format for h.263 coder) */
/*	convert(dataPtr, buf); 

	fwrite(buf, YUV_411_FRAME_SIZE, 1, file); */
	
	/* Finished with frame, unlock the buffer */
	vlPutFree(svr, buffer);
     }
    
    close_capture();
    fclose(file);
}

void init_capture()
  {
     VLDev dev;

    /* Connect to the daemon */
    if (!(svr = vlOpenVideo(""))) 
	error_exit();

    /* Set up a drain node in memory */
    drn = vlGetNode(svr, VL_DRN, VL_MEM, VL_ANY);
    
    /* Set up a source node on any video source  */
    src = vlGetNode(svr, VL_SRC, VL_VIDEO, 1);

    /* Create a path using the first device that will support it */
    path = vlCreatePath(svr, VL_ANY, src, drn); 

    /* Set up the hardware for and define the usage of the path */
    if ((vlSetupPaths(svr, (VLPathList)&path, 1, VL_SHARE, VL_SHARE)) < 0)
	error_exit();

    val.intVal = 10;
    vlSetControl(svr, path, drn, VL_RATE, &val);

/*    val.fractVal.numerator = 1;
    val.fractVal.denominator = 2;
    vlSetControl(svr, path, drn, VL_ZOOM, &val);
*/
/*    val.xyVal.x = 176;
    val.xyVal.y = 144;
    vlSetControl(svr, path, drn, VL_SIZE, &val);
*/
    /* Set the packing to RGB */
    val.intVal = VL_PACKING_RGB_8;
    vlSetControl(svr, path, drn, VL_PACKING, &val);
    
    /* Get the video size */
    vlGetControl(svr, path, drn, VL_SIZE, &val);
    xsize = val.xyVal.x;
    ysize = val.xyVal.y;

    vlGetControl(svr, path, drn, VL_RATE, &val);
     printf("Rate: %d\n", val.intVal);

    /* Create and register a buffer for 1 frame */
    buffer = vlCreateBuffer(svr, path, drn, 1);
    if (buffer == NULL)
       error_exit();	
    vlRegisterBuffer(svr, path, drn, buffer);
    
    /* Begin the data transfer */
    if (vlBeginTransfer(svr, path, 0, NULL))
       error_exit();
 }

close_capture()
  {
    /* End the data transfer */
    vlEndTransfer(svr, path);
     
    /* Cleanup before exiting */
    vlDeregisterBuffer(svr, path, drn, buffer);
    vlDestroyBuffer(svr, buffer);
    vlDestroyPath(svr, path);
    vlCloseVideo(svr);
  }

void init_dpy()
  {
    foreground();
        
    /* Set up and open a GL window to display the data */
    prefsize(xsize,ysize);
    win = winopen("Simpleccapt Window");
    RGBmode();
    pixmode(PM_TTOB, 1);
    gconfig();
  }
